<?php

defined('ABSPATH') || exit;

use WcMipConnector\Controller\ApiController;
use WcMipConnector\Service\ZoneService;
use WcMipConnector\Controller\ConfigurationController;
use WcMipConnector\View\Assets\Assets;
use WcMipConnector\Service\LoggerService;

$loggerService = null;

try {
    $loggerService = new LoggerService();
} catch (\Throwable $exception) {
    \error_log('Logger initialization: '.$exception->getMessage());
}

try {
    $loggerService = new LoggerService();
    $classMipconnector = new WcMipconnector();
    $assets = new Assets();
} catch (\Throwable $exception) {
    if ($loggerService) {
        $loggerService->critical(__FILE__.' Hooks initialization: '.$exception->getMessage());
    }
}

try {
    $apiController = new ApiController();

    add_action('init', [$apiController, 'addEndpoints']);
    add_action('init', [$apiController, 'addActions']);
} catch (\Throwable $exception) {
    if ($loggerService) {
        $loggerService->critical(__FILE__.' API initialization: '.$exception->getMessage());
    }
}

try {
    $configurationController = new ConfigurationController();
    add_action('admin_bar_menu', [$configurationController, 'initializeAdminViews'] );
    add_action('admin_menu', [$configurationController, 'loadMenu']);
    add_action('init', [$configurationController, 'loadTranslations']);
} catch (\Throwable $exception) {
    if ($loggerService) {
        $loggerService->critical(__FILE__.' Configuration initialization: '.$exception->getMessage());
    }
}

try {
    $carrierService = new ZoneService();
    add_filter('woocommerce_package_rates', [$carrierService, 'shippingMethodsAvailable'], 10);
    add_filter('woocommerce_cart_shipping_method_full_label', [$carrierService, 'modifyShippingLabel'], 10, 2);
    add_filter('woocommerce_after_shipping_rate', [$carrierService, 'findDataToProcess'], 10, 3);
    add_action('woocommerce_after_calculate_totals', [$carrierService, 'getTotalCart'], 10, 1);
} catch (\Throwable $exception) {
    if ($loggerService) {
        $loggerService->critical(__FILE__.' Shipping rates initialization: '.$exception->getMessage());
    }
}