<?php

namespace WcMipConnector\Manager;

defined('ABSPATH') || exit;

use WcMipConnector\Enum\WooCommerceApiMethodTypes;
use WcMipConnector\Exception\WooCommerceApiExceptionInterface;
use WcMipConnector\Service\WoocommerceApiAdapterService;

class ZoneManager
{
    /** @var WoocommerceApiAdapterService */
    protected $apiAdapterService;
    /** @var TaxesManager */
    protected $taxesManager;

    public function __construct()
    {
        $this->apiAdapterService = new WoocommerceApiAdapterService();
        $this->taxesManager = new TaxesManager();
    }

    /**
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function findAllZones(): array
    {
        return $this->apiAdapterService->getItems(WooCommerceApiMethodTypes::TYPE_SHIPPING_ZONES);
    }

    /**
     * @param array $zonesIndexedById
     *
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function createZones(array $zonesIndexedById): array
    {
        return $this->apiAdapterService->createItem(WooCommerceApiMethodTypes::TYPE_SHIPPING_ZONES, $zonesIndexedById);
    }

    /**
     * @param int $zoneId
     *
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function findShippingZoneByZoneId(int $zoneId): array
    {
        $filters = ['zone_id' => $zoneId];

        return $this->apiAdapterService->getItems(WooCommerceApiMethodTypes::TYPE_SHIPPING_ZONES_METHOD, $filters);
    }

    /**
     * @param int   $zoneId
     * @param array $shippingZone
     *
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function createShippingZones(int $zoneId, array $shippingZone): array
    {
        $shippingZone['zone_id'] = $zoneId;

        return $this->apiAdapterService->createItem(WooCommerceApiMethodTypes::TYPE_SHIPPING_ZONES_METHOD, $shippingZone);
    }

    /**
     * @param int $zoneId
     * @param int $methodId
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function enableShippingZoneMethodByZoneIdAndMethodId(int $zoneId, int $methodId): array
    {
        $data = [
            'enabled' => true,
            'zone_id' => $zoneId,
            'method_id' => $methodId,
        ];

        return $this->apiAdapterService->updateItem(WooCommerceApiMethodTypes::TYPE_SHIPPING_ZONES_METHOD, $data);
    }
}