<?php

namespace WcMipConnector\Service;

use WcMipConnector\Enum\MipWcConnector;
use WcMipConnector\Factory\PublicationOptionsFactory;
use WcMipConnector\Manager\ConfigurationOptionManager;
use WcMipConnector\Client\MIP\Customer\Service\SellingChannelService as SellingChannelServiceMIP;

defined('ABSPATH') || exit;

class PublicationOptionsService
{
    /** @var SellingChannelServiceMIP */
    private $sellingChannelServiceMIP;

    public function __construct()
    {
        $accessToken = ConfigurationOptionManager::getAccessToken();
        $this->sellingChannelServiceMIP = SellingChannelServiceMIP::getInstance($accessToken);
    }

    /**
     * @return int
     */
    public function getConversionFactor(): int
    {
        $cacheId = CacheService::generateCacheKey(__METHOD__);
        if (CacheService::getInstance()->has($cacheId)) {
            return CacheService::getInstance()->get($cacheId);
        }

        if ($this->isUpdatePublicationOptionRequired()) {
            $customerPublicationOptions = $this->sellingChannelServiceMIP->getCustomerPublicationOptions();
            $publicationOptions = PublicationOptionsFactory::getInstance()->create($customerPublicationOptions->conversionFactor, date('Y-m-d H:i:s'));
            ConfigurationOptionManager::setPublicationOptions($publicationOptions);
            $conversionFactor = $customerPublicationOptions->conversionFactor;
        } else {
            $customerPublicationOptions = \json_decode(ConfigurationOptionManager::getPublicationOptions(), true);
            $conversionFactor = $customerPublicationOptions['conversionFactor'];
        }

        CacheService::getInstance()->set($cacheId, $conversionFactor);

        return $conversionFactor;
    }

    /**
     * @return bool
     */
    public function isUpdatePublicationOptionRequired(): bool
    {
        $publicationOptionsStore = ConfigurationOptionManager::getPublicationOptions();

        if (!$publicationOptionsStore
            || !\array_key_exists('LastRequest', \json_decode($publicationOptionsStore, true))
        ) {
            return true;
        }

        $lastRequest = \json_decode($publicationOptionsStore, true);

        if (!$lastRequest['LastRequest']) {
            return true;
        }

        try {
            $dateNow = new \DateTime('now', new \DateTimeZone('UTC'));
            $dateLastRequest = new \DateTime($lastRequest['LastRequest'], new \DateTimeZone('UTC'));

            return $dateNow->diff($dateLastRequest)->days >= 1;
        } catch (\Exception $exception) {
            return true;
        }
    }
}